;----------------------------------------------------------------------
;put const-string to LCD
lcdstring:				
	mov tmppointer,#0		;use tmppointer for counting
lcdstring0:
	cjne tmppointer,#11h,lcdstring1
	jmp lcdstringret
lcdstring1:
	mov a,tmppointer
	movc a,@a+DPTR			;calculate text-index
	cjne a,#ENDE,lcdstring2		;check for end-byte
	jmp lcdstringret
lcdstring2:
	mov b,#LCD_DataWR		;put byte to LCD
	call out_io
	jnb step_walk,lcdstring3
	call wait
lcdstring3:
	call shortwait
	call LCDBusy
	jc lcdstring3			;wait until LCD is ready
	inc tmppointer
	cjne tmppointer,#8h,lcdstring0	;jump in next LCD-RAM-bank
	mov a,#ADR_2LCD			;if 8 byte reached
	call setcursor
lcdstring4:
	call shortwait
	call LCDBusy
	jc lcdstring4			;wait until LCD-cursor is set
	jmp lcdstring0
lcdstringret:
	clr step_walk
	ret

;----------------------------------------------------------------------
;put string(lcd_buffer) to LCD
WriteLCD_Array:
	mov tmppointer,#lcd_buffer
WriteLCD0:
	mov a,@tmppointer
	cjne a,#ENDE,WriteLCD1	;check for end
	jmp WriteLCDRet
WriteLCD1:
	mov a,@tmppointer
	mov b,#LCD_DataWR
	call out_io
	inc tmppointer
	jnb step_walk,WriteLCD2
	call wait
WriteLCD2:
	call shortwait
	call LCDBusy
	jc WriteLCD2	;check if busy
	mov a,tmppointer
	clr c
	subb a,#lcd_buffer              ;Array_overrun?
	cjne a,#8,WriteLCDoverrun
	mov a,#ADR_2LCD
	call setcursor
WriteLCD3:
	call shortwait
	lcall LCDBusy
	jc WriteLCD3
	jmp WriteLCD0
WriteLCDoverrun:
	cjne a,#11h,WriteLCD0 
WriteLCDRet:
	clr step_walk
	ret

;--------------------------------------------
;put byte to LCD
;acc -> data 
write_char:
	push dpl
	push dph
	mov dpl,#LCD_DataWR
	anl dpl,#0fh
	mov dph,#LCD_DataWR
	anl dph,#0f0h
	movx @DPTR,a
	call getCursor
	cjne a,#ADR_END_1LCD,wr_ch_end2
	mov a,#ADR_2LCD			;cursor to 2.bank
	call setcursor
	jmp wr_char_ret
wr_ch_end2:
	cjne a,#ADR_END_2LCD,wr_char_ret
	mov a,#ADR_1LCD			;cursor to home
	call setcursor
	jmp wr_char_ret
wr_char_ret:
	pop dph
	pop dpl
	ret

;--------------------------------------------
;reads LCD-Contr. Carry=1 if busy
LCDBusy:        

	;xxl    anlernative code  -> only short waitcycle
	;call shortwait	;wait a little time
	;clr c		;LCD should be ready
	;ret

	push dpl
	push dph
	mov dpl,#LCD_RegRD
	anl dpl,#0fh
	mov dph,#LCD_RegRD
	anl dph,#0f0h
	movx a,@DPTR
	clr c
	anl a,#80h
	jz LCDBusyRet
	setb c
LCDBusyRet:
	pop dph
	pop dpl
	ret
	
;--------------------------------------------
; set cursor in LCD-module
; in acc
setcursor:
	push dpl
	push dph
	mov dpl,#LCD_RegWR
	anl dpl,#0fh
	mov dph,#LCD_RegWR
	anl dph,#0f0h
	orl a,#80h	;set command-bit
	movx @DPTR,a	;set cursor
	pop dph
	pop dpl
	ret

;--------------------------------------------
;read RAM-addr of LCD and return it in acc (0-127)
;out acc
getCursor:
	push dpl
	push dph
	mov dpl,#LCD_RegRD
	anl dpl,#0fh
	mov dph,#LCD_RegRD
	anl dph,#0f0h
	movx a,@DPTR
	anl a,#7Fh	;clear busy-flag
	pop dph
	pop dpl
	ret
;--------------------------------------------
; set cursor home and clear LCD-module
clearLCD:
	push dpl
	push dph
	mov a,#1h               ;homecursor and clear

	mov dpl,#LCD_RegWR
	anl dpl,#0fh
	mov dph,#LCD_RegWR
	anl dph,#0f0h
	movx @DPTR,a
clear_busy:
	call shortwait
	call LCDBusy
	jc clear_busy
	pop dph
	pop dpl
        ret

;--------------------------------------------
; wait a little bit
shortwait:
	push acc
	mov a,#0
shortwait0:
	nop
	nop
	nop
	djnz acc,shortwait0
	pop acc
	ret

;--------------------------------------------
; wait a little bit longer 
wait:
	mov a,#waitconst
wait0:
	call shortwait
	djnz acc,wait0
	ret

;--------------------------------------------
;acc -> data 
;b   -> IO-high-adresse
out_io:
	push dpl
	push dph
	mov dpl,b
	anl dpl,#0fh
	mov dph,b
	anl dph,#0f0h
	movx @DPTR,a
	pop dph
	pop dpl
	ret

;--------------------------------------------
;return data in acc 
;b   -> IO-high-adresse
in_io:
	push dpl
	push dph
	mov dpl,b
	anl dpl,#0fh
	mov dph,b
	anl dph,#0f0h
	movx a,@DPTR
	pop dph
	pop dpl
	ret

;-----------------------------------------------------------------
;init LCD-modul
lcd_init:
	mov tmppointer,#2
init_again:
	call shortwait
	mov a,#00111000b		;8bit-mode one line 5x7dots
	mov b,#LCD_RegWR
	lcall out_io
	lcall out_io
	call shortwait
	djnz tmppointer,init_again	;call 2x init-sequence

	jb blick_cursor,b_cursor	
	mov a,#00001100b      		;LCD on no Cursor
	jmp after_cur
b_cursor:
	mov a,#00001101b      		;LCD on no Cursor
after_cur:
	mov b,#LCD_RegWR
	lcall out_io
	lcall shortwait
	mov a,#00000001b		;clear lcd
	mov b,#LCD_RegWR
	lcall out_io
	lcall shortwait
	mov a,#00000110b		;set corsor-direction auto-dec
	mov b,#LCD_RegWR
	lcall out_io
	lcall shortwait
	clr blick_cursor
	ret

